/******************************************************************************
 glclass.h---------------------------------------------------------------------
 Este arquivo contm todas as informaes e cdigos necessrios para a parte
 OpenGL do programa, incluindo controle de Janela, Mouse e Teclado.
 *****************************************************************************/




#pragma once





#include <stdlib.h>
#include <GL/glut.h>
#include <cmath>
#include <cstring>
#include "glauxiliar.h"
#include "cores.h"
#include "mapa.h"
#include "matriz.h"
#include "tiffsave.h"



using namespace std;





/*******************************************************************************
 Inicio da Declarao das Funes.
 *******************************************************************************/
/* Funes sem relao */

char *imageName();




/* Funes responsveis pelo Display */
void display(void); // Funo OpenGL que serve como leitor da mquina de estado
void displayBolacha(void); // Funo OpenGL que exibe a Bolacha inteira
//void displayBolachaZoom(void); // Funo OpenGL que d o Zoom normal (no fechado)
void displayCorte(void); // Funo OpenGL que exibe um Setor da Bolacha -- Essa funo sera alterada
void displayReta(void); // Funo OpenGL que exibe um Corte de um Setor da Bolacha
void displayZoom(void);
void displayDados(void);
void displayMenu(void); 






/* Funes responsveis por desenhar objetos para display */
void drawCidades(); // Funo de desenho das cidades
void drawEscala(void); // Desenha escala geogrfica
void drawPosition(); // Imprime na tela a posio do mouse
void drawQuadrado(int, int, int); // Desenha o equivalente  um valor de dado do arquivo
void drawQuadradoSimples(int , int, int); // Desenha o equivalente  um valor de dado do arquivo, mas simples
void drawString(char *, int); // Desenha uma String de Texto
void drawSelecao(float, float, float, float);





void drawQuadradoSuave(float x, float y, float xp, float yp, int cor);
void drawQuadradoReal(float, float, float, float, int); // Desenha o equivalente  um valor de dado do arquivo, mas simples
void drawCores();






/* Funes de Manuteno da Janela */
void resize(int, int); // Funo OpenGL para redimensionamento de Janela





/* Funes responsveis pelo Menu */
void menuMain(int); // Funo OpenGL de Menu do Mouse
void menuSubGrafico(int); // SubMenu
void menuSubGraficoSelect(int);
void menuSubMedidas(int);
void menuSubDadosVoltar(int);
void menuSubDadosCores(int);
void menuDados(int);
void menuSubCores(int);
void menuSubCorQuantidade(int);
void menuTiff(int);
void menuSalvar(int);
void subLeituraQualidade(int);






/* Funes responsveis pelo Mouse */
void mouse(int, int, int, int); // Funo OpenGL de controle do Mouse
void mousePosicao(int, int); // funo do mouse passivo, retorna a posio do mesmo na tela
void mouseMovimento(int, int); // funo do mouse ativo, retorna a posio do mesmo na tela







/* Funes responsveis pelo Teclado */
void keyboard(int, int, int);
void teclado(unsigned char, int, int);








/* Outras funes */
void selecaoTermina(float, float);
void selecaoInicia(float, float);
void resetFigure();







/*******************************************************************************
 Inicio da Declarao das Variveis e Objetos
 *******************************************************************************/

/* Enumeraes */

enum nivelColorido { PRETO_E_BRANCO, BRANCO_E_PRETO, COLORIDO_E_PRETO, COLORIDO_E_BRANCO };
nivelColorido nivelColorido = COLORIDO_E_PRETO;





enum medidas { MILHAS, KILOMETROS };
medidas unidadeMedidas = KILOMETROS;


enum qualidade { BOA, RUIM, PIOR };
qualidade qualidadeImagem = RUIM;



/* Controle de Janela */
unsigned int height = 640; // altura da janela
unsigned int width = 640; // largura da janela




unsigned int retaInicio = 0; // Primeiro ponto da linha/coluna da reta
unsigned int retaFim = 0; // ltimo ponto da linha/coluna da reta
unsigned int contaPontos = 0; // Conta quantos pontos foram criados pelo usurio
unsigned int contaPontosReta = 0; // Conta quantos pontos foram criados pelo usurio para a reta





/* Variaveis responsveis pelo posicionamento do mouse */
unsigned int Gazimute = 0; // azimute global para posio do mouse
float Graio = 0, Gintensidade = -50; // globais para posio do mouse





/* Controle de Menu */
int idMenuMain; // ID do Menu
int idMenuSubGrafico; // ID do Submenu
int idMenuSubGraficoSelect; // ID do Submenu
int idMenuDados; // ID do menu da tela de Dados
int idMenuSubDadosCores; // ID do menu da tela de Dados para escolher cores
/*int idMenuSubDadosVoltar; // ID do menu da tela de Dados para voltar*/
int idMenuSubMedidas; // ID do menus da tela de Dados para escolher unidades de medida
int idMenuSubCores;
int idMenuSalvar;
int idMenuTiff;
int idSubCorQuantidade;
int idSubLeituraQualidade;





GLfloat xZeroOld = 0.0;
GLfloat yZeroOld = 0.0;
GLfloat escalaGlobalOld = 1.0;
/* Ajustes */
int primeiroRaio = 4;
int primeiroAngulo = 0;
int ajusteR = 0;
int ajusteA = 0;
int qualidadeR = 4; // valores:= 4 = pior e rapido, 2 = medio, 1 = melhor e lento
/* Ajuste de Translao */
const float cidadePPRX = 0.54;
const float cidadePPRY = -0.025;




unsigned int contadorDeImagens = 0; // conta o nmero da imagem em questo




/* janelas e subjanelas*/
int janelaPrincipal;
int janelaMenu;





/*******************************************************************************
 Inicio da Definio das Funes.
 *******************************************************************************/
	









/*******************************************************************************
 *******************************************************************************
 *******************************************************************************
                         Funes de Controle do OpenGL
 *******************************************************************************
 *******************************************************************************
 *******************************************************************************/




void resize(int w, int h) { 
	width = w;
	height = h;
	GLfloat nRange = 1.0f;
	
	// evita diviso por zero
	if(h == 0) {
		h = 1;
	} // if(h == 0)
	



	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	if(w <= h) {
		glOrtho(-nRange, nRange, -nRange*h/w, nRange*h/w, -nRange, nRange);
	} // if(w <= h)
	else {
		glOrtho(-nRange*w/h, nRange*w/h, -nRange, nRange, -nRange, nRange);
	} // else 
		
	glMatrixMode(GL_MODELVIEW);
} // void resize(int w, int h)












/*******************************************************************************
 *******************************************************************************
 *******************************************************************************
                     Funes responsveis pelo Display
 *******************************************************************************
 *******************************************************************************
 *******************************************************************************/






void display(void) {
	if (glutLayerGet(GLUT_OVERLAY_DAMAGED)) {
		glMatrixMode(GL_MODELVIEW);
		if (qualidadeImagem == BOA) {
			glShadeModel(GL_SMOOTH);
		}
		else {
			glShadeModel(GL_FLAT);
		}

		glClearColor(0.6, 0.6, 0.6, 0.0); // Fundo
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	} // if (glutLayerGet(GLUT_OVERLAY_DAMAGED)) 




	switch (estado) {
		case ESPERA:
		case SELECIONA_FATIA:
			displayBolacha();
			break;
		case EXIBE_ZOOM:
		case SELECIONA_CORTE_DE_ZOOM:
		case SELECIONA_ZOOM:
			displayZoom();
			break;
		case EXIBE_FATIA:
		case SELECIONA_ZOOM_DE_CORTE:
			displayCorte();
			break;
		case SELECIONA_CENTRO:
			switch (estadoAnterior) {
				case EXIBE_ZOOM:
				case SELECIONA_CORTE_DE_ZOOM:
				case SELECIONA_ZOOM:
					displayZoom();
					break;
				case SELECIONA_GRAFICO:
				case EXIBE_FATIA:
					displayCorte();
					break;
			}
			break;



		case SELECIONA_NUMEROS:
				switch (estadoAnterior) {
				case ESPERA:
				case SELECIONA_FATIA:
					displayBolacha();
					break;
				case EXIBE_ZOOM:
				case SELECIONA_CORTE_DE_ZOOM:
				case SELECIONA_ZOOM:
					displayZoom();
					break;
				case SELECIONA_GRAFICO:
				case EXIBE_FATIA:
					displayCorte();
					break;
			}
			break;



		case SELECIONA_GRAFICO:
			switch (estadoAnterior) {
				case ESPERA:
				case SELECIONA_FATIA:
					displayBolacha();
					break;
				case EXIBE_ZOOM:
				case SELECIONA_CORTE_DE_ZOOM:
				case SELECIONA_ZOOM:
					displayZoom();
					break;
				case SELECIONA_GRAFICO:
				case EXIBE_FATIA:
					displayCorte();
					break;
			}
			break;


		case EXIBE_GRAFICO:
			displayReta();
			break;


		case EXIBE_NUMEROS:
			displayDados();
			break;


		default:
			estado = estadoAnterior;
			break;


	} // switch (estado) 






	if (estado == EXIBE_NUMEROS) {
		if (nivelColorido == COLORIDO_E_PRETO || nivelColorido == BRANCO_E_PRETO) { 
			glColor3f(1.0F, 1.0F, 1.0F);
		}
		else {
			glColor3f(0.0F, 0.0F, 0.0F);
		}





		glRasterPos2f(-0.99, -0.99);
		drawString("Use as teclas direcionais do teclado para mover os dados.", 0);
		glutSwapBuffers();
		//glutPostRedisplay();
	}

	else if (estado == EXIBE_GRAFICO) {
		glColor3f(0.0F, 0.0F, 0.0F);
		glBegin(GL_POLYGON);
			glVertex3f(-1.0, -0.95, 0.0);
			glVertex3f(1.0, -0.95, 0.0);
			glVertex3f(1.0, -1.0, 0.0);
			glVertex3f(-1.0, -1.0, 0.0);
		glEnd();
		drawPosition();
		glRasterPos2f(-0.12, -0.99);
		drawString("Use as teclas direcionais do teclado para mover os dados.", 0);
		glutSwapBuffers();
	}
	else {
		glColor3f(0.0F, 0.0F, 0.0F);
		glBegin(GL_POLYGON);
			glVertex3f(-1.0, -0.95, 0.0);
			glVertex3f(-0.12, -0.95, 0.0);
			glVertex3f(-0.12, -1.0, 0.0);
			glVertex3f(-1.0, -1.0, 0.0);
		glEnd();
		drawPosition();
	
		glutSwapBuffers();
	}


} // void display(void) 






void displayBolacha(void) {


	/* Escolhe qual matriz mostrar */
	unsigned int tmpxyz = matriz.raioMaximo/qualidadeR;



	if (qualidadeImagem == BOA) { 
		for (int i = 0; i < ANGULO; i++) { // de 0 at 359
			for (unsigned int r = matriz.raioMinimo/qualidadeR; r < tmpxyz; r++) {// de 1 at 1998
				drawQuadrado(qualidadeR, i, r);
			} // for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++)
		} // for(int i = 0; i < ANGULO; i++)
	}
	else if (qualidadeImagem == RUIM) {
		for(int i = 0; i < ANGULO; i++) { // de 0 at 359
			for (unsigned int r = matriz.raioMinimo/qualidadeR; r < tmpxyz; r++) {// de 1 at 1998
				drawQuadradoSimples(qualidadeR, i, r);
			} // for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++)
		} // for(int i = 0; i < ANGULO; i++)
	}
	else {
		// glPointSize(1.0);
		glPointSize(escalaGlobal);
		glBegin(GL_POINTS);
			for(int i = 0; i < ANGULO; i++) { // de 0 at 359
				for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++) {// de 1 at 1998
					getNewColor(1, i, r); // recebe a cor da posio [i][r] da matriz
					glVertex3f(eixoX(azToPol(i), r), eixoY(azToPol(i), r), 0.0f);
				} // for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++)
			} // for(int i = 0; i < ANGULO; i++)
		glEnd();
	}





	/**************************
	 * Desenha escala e o mapa
	 **************************/




	drawEscala();
	if (lugar == PPR) {
		glPushMatrix();
			glTranslatef(cidadePPRX, cidadePPRY, 0.0);
			drawMapa();
			drawCidades();
		glPopMatrix();
	}
	else if (lugar == BRU) {
		glPushMatrix();
			drawMapa();
			drawCidades();
		glPopMatrix();
	}





	if (tracking == true) {
		drawSelecao(xSelecaoInicial, ySelecaoInicial, xSelecaoAtual, ySelecaoAtual);
		// chamaZoom();
	}



} // void displayBolacha(void)







void displayZoom(void) {
	glPushMatrix();
		glTranslatef(-xZero*escalaGlobal, -yZero*escalaGlobal, 0.0);
		glScalef(escalaGlobal, escalaGlobal, 1.0);
		int sizeR = qualidadeR;
		//if (escalaGlobal < 5.0)
		// sizeR = 4;
		//else if (escalaGlobal < 10.0)
		//	sizeR = 2;
		//else sizeR = 1;





		unsigned int tmpxyz = matriz.raioMaximo/sizeR;
		if (qualidadeImagem == BOA) { 
			for(int i = 0; i < ANGULO; i++) { // de 0 at 359
				for(unsigned int r = matriz.raioMinimo/sizeR; r < tmpxyz; r++) {// de 1 at 1998
					drawQuadrado(sizeR, i, r);
				} // for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++)
			} // for(int i = 0; i < ANGULO; i++)
		}
		else if (qualidadeImagem == RUIM) {
			for(int i = 0; i < ANGULO; i++) { // de 0 at 359
				for(unsigned int r = matriz.raioMinimo/sizeR; r < tmpxyz; r++) {// de 1 at 1998
					drawQuadradoSimples(sizeR, i, r);
				} // for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++)
			} // for(int i = 0; i < ANGULO; i++)
		}
		else {
			glPointSize(escalaGlobal);
			glBegin(GL_POINTS);
				for(int i = 0; i < ANGULO; i++) { // de 0 at 359
					for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++) {// de 1 at 1998
						getNewColor(1, i, r); // recebe a cor da posio [i][r] da matriz
						glVertex3f(eixoX(azToPol(i), r), eixoY(azToPol(i), r), 0.0f);
					} // for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++)
				} // for(int i = 0; i < ANGULO; i++)
			glEnd();
		}
		





		drawEscala();
		if (lugar == PPR) {
			glPushMatrix();
				glTranslatef(cidadePPRX, cidadePPRY, 0.0);
				drawMapa();
				drawCidades();
			glPopMatrix();
		}
		else if (lugar == BRU) {
			glPushMatrix();
				drawMapa();
				drawCidades();
			glPopMatrix();
		}
	glPopMatrix();
	






	if (tracking == true) {
				drawSelecao(xSelecaoInicial, ySelecaoInicial, xSelecaoAtual, ySelecaoAtual);
				// chamaZoom();
	}
} // void displayBolacha(void)






void displayCorte(void) {
	unsigned int raioI = pontos[0][1], raioF = pontos[1][1];
	
	if(raioI > raioF) {
		unsigned int k = raioI;
		raioI = raioF;
		raioF = k;
	} // if(raioI > raioF) 







	unsigned int anguloI = azToPol(pontos[0][0]), anguloF = azToPol(pontos[1][0]);







	if (anguloI == anguloF) {
		return;
		}






	if( (360 + anguloI - anguloF)%360 < (360 + anguloF - anguloI)%360 ) {
		unsigned int k = anguloI;
		anguloI = anguloF;
		anguloF = k;
	} // if(anguloI > anguloF)
	




	glPushMatrix();
		GLfloat escala = 2.0/((xIntervalo > yIntervalo?xIntervalo:yIntervalo));
		escalaGlobal = escala;
		glTranslatef(-xZero*escala, -yZero*escala, 0.0);
		glScalef(escala, escala, 1.0);
		
		// for(int i = 0; i < ANGULO; i++) { // de 0 at 359
			// for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++) {
				// de 1 at 1998
		if (qualidadeImagem == BOA) { 
			for(unsigned int i = anguloI; i <= anguloF; i++) {// de 0 at 359
				for(unsigned int r = raioI; r <= raioF; r++) {
					drawQuadrado(1, i, r);
				} // for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++)
			} // for(int i = 0; i < ANGULO; i++)
		}
		else if (qualidadeImagem == RUIM) {
			for(unsigned int i = anguloI; i <= anguloF; i++) {// de 0 at 359
				for(unsigned int r = raioI; r <= raioF; r++) {
					drawQuadradoSimples(1, i, r);
				} // for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++)
			} // for(int i = 0; i < ANGULO; i++)
		}
		else {
			glPointSize(escalaGlobal);
			glBegin(GL_POINTS);
				for(unsigned int i = anguloI; i <= anguloF; i++) {// de 0 at 359
					for(unsigned int r = raioI; r <= raioF; r++) {
						getNewColor(1, i, r); // recebe a cor da posio [i][r] da matriz
						glVertex3f(eixoX(azToPol(i), r), eixoY(azToPol(i), r), 0.0f);
					} // for(unsigned int r = matriz.raioMinimo; r < matriz.raioMaximo; r++)
				} // for(int i = 0; i < ANGULO; i++)
			glEnd();
		}







		drawEscala();
		if (lugar == PPR) {
			glPushMatrix();
				glTranslatef(cidadePPRX, cidadePPRY, 0.0);
				drawMapa();
				drawCidades();
			glPopMatrix();
		}
		else if (lugar == BRU) {
			glPushMatrix();
				drawMapa();
				drawCidades();
			glPopMatrix();
		}


	glPopMatrix();






	if (tracking == true) {
		drawSelecao(xSelecaoInicial, ySelecaoInicial, xSelecaoAtual, ySelecaoAtual);
		// chamaZoom();
	}



} // void displayCorte(void)








void displayReta(void) {
	switch (nivelColorido) {
			case PRETO_E_BRANCO:
		case COLORIDO_E_BRANCO:
			glClearColor(1.0, 1.0, 1.0, 0.0); // Fundo
			glColor3f(0.0,0.0,0.0);
			break;
		case BRANCO_E_PRETO:
		case COLORIDO_E_PRETO:
			glClearColor(0.0, 0.0, 0.0, 0.0); // Fundo
			glColor3f(1.0,1.0,1.0);
			break;
		default:
			break;
	}
	glMatrixMode(GL_MODELVIEW);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);




	glLineWidth(1.0);
	



	glRasterPos2f(-1.0, -0.45);
	drawString("0 dBZ", 0);
	glBegin(GL_LINES); // Linha do Zero
		glVertex3f(-1.0, -0.5, 0.0);
		glVertex3f(1.0, -0.5, 0.0);
	glEnd();
	glRasterPos2f(-1.0, -0.05);
	drawString("20 dBZ", 0);
	glBegin(GL_LINES); // Linha do 20
		glVertex3f(-1.0, -0.1, 0.0);
		glVertex3f(1.0, -0.1, 0.0);
	glEnd();
	glRasterPos2f(-1.0, 0.35);
	drawString("40 dBZ", 0);
	glBegin(GL_LINES); // Linha do 40
		glVertex3f(-1.0, 0.3, 0.0);
		glVertex3f(1.0, 0.3, 0.0);
	glEnd();
	glRasterPos2f(-1.0, 0.75);
	drawString("60 dBZ", 0);
	glBegin(GL_LINES); // Linha do 60
		glVertex3f(-1.0, 0.7, 0.0);
		glVertex3f(1.0, 0.7, 0.0);
	glEnd();






	glColor3f(0.5,0.5,0.5);
	glRasterPos2f(-1.0, -0.25);
	drawString("10 dBZ", 0);
	glBegin(GL_LINES); // linha do 10
		glVertex3f(-1.0, -0.3 , 0.0);
		glVertex3f(1.0, -0.3 , 0.0);
	glEnd();
	glRasterPos2f(-1.0, 0.15);
	drawString("30 dBZ", 0);
	glBegin(GL_LINES); // linha do 30
		glVertex3f(-1.0, 0.1, 0.0);
		glVertex3f(1.0, 0.1, 0.0);
	glEnd();
	glRasterPos2f(-1.0, 0.55);
	drawString("50 dBZ", 0);
	glBegin(GL_LINES); // linha do 50
		glVertex3f(-1.0, 0.5, 0.0);
		glVertex3f(1.0, 0.5, 0.0);
	glEnd();
	glRasterPos2f(-1.0, 0.95);
	drawString("70 dBZ", 0);
	glBegin(GL_LINES); // linha do 70
		glVertex3f(-1.0, 0.9, 0.0);
		glVertex3f(1.0, 0.9, 0.0);
	glEnd();






	if (isLinha == true) {
		if (selecao[0]+ajusteR < matriz.getRaioInicial() ) {
			ajusteR = matriz.getRaioInicial() - selecao[0];
		}
		else if (selecao[1]+ajusteR > matriz.getRaioFinal()) {
			ajusteR = matriz.getRaioFinal() - selecao[1];
		}




		double maior = -100.0, menor = 100.0;




		for (unsigned int i = selecao[0]; i <= selecao[1]; i++) {
			// era int tmp
			double tmp = matriz.getTabela((360+selecao[2]+ajusteA)%360, i+ajusteR);
			maior = tmp>maior?tmp:maior;
			menor = tmp<menor?(tmp>-25.0?tmp:menor):menor;
		} // for (unsigned int i = selecao[0]; i <= selecao[1]; i++)




		double diferenca = maior - menor;
		diferenca = diferenca<0?-diferenca:diferenca;
		



		double tamanho = 2.0/(selecao[1]-selecao[0]);
		



		glBegin(GL_LINE_STRIP);
		unsigned int finalSelecao = selecao[1]-selecao[0];
		switch (nivelColorido) {
			case COLORIDO_E_PRETO:
			case COLORIDO_E_BRANCO:
				for (unsigned int i = 0; i <= finalSelecao; i++) {
						getLineColor((360+selecao[2]+ajusteA)%360, selecao[0]+i+ajusteR);
						glVertex3f(-1.0 + i*tamanho, -1.0 + (matriz.getTabela((360+selecao[2]+ajusteA)%360, selecao[0]+ajusteR+i)+25)/50.0, 0.0);
				} // for (unsigned int i = 0; i <= selecao[1]-selecao[0]; i++)
				break;
			case BRANCO_E_PRETO:
				glColor3f(1.0,1.0,1.0);
				for (unsigned int i = 0; i <= finalSelecao; i++) {
						glVertex3f(-1.0 + i*tamanho, -1.0 + (matriz.getTabela((360+selecao[2]+ajusteA)%360, selecao[0]+ajusteR+i)+25)/50.0, 0.0);
				} // for (unsigned int i = 0; i <= selecao[1]-selecao[0]; i++)
				break;
			case PRETO_E_BRANCO:
				glColor3f(0.0,0.0,0.0);
				for (unsigned int i = 0; i <= finalSelecao; i++) {
						glVertex3f(-1.0 + i*tamanho, -1.0 + (matriz.getTabela((360+selecao[2]+ajusteA)%360, selecao[0]+ajusteR+i)+25)/50.0, 0.0);
				} // for (unsigned int i = 0; i <= selecao[1]-selecao[0]; i++)
				break;
			default:
				break;
		}





		glEnd();
	} // if (isLinha == true)
	else {
		double maior = -100.0, menor = 100.0;
		
		if (selecao[2]+ajusteR < matriz.getRaioInicial() ) {
			ajusteR = matriz.getRaioInicial() - selecao[2];
		}
		else if (selecao[2]+ajusteR > matriz.getRaioFinal()) {
			ajusteR = matriz.getRaioFinal() - selecao[2];
		}







		for (unsigned int i = 0; i <= (360+selecao[1]-selecao[0])%360; i++) {
			//tmp era int
			double tmp = matriz.getTabela((360+selecao[0]+i+ajusteA)%360, selecao[2]+ajusteR);
			maior = tmp>maior?tmp:maior;
			menor = tmp<menor?(tmp>-25.0?tmp:menor):menor;
		} // for (unsigned int i = 0; i <= (360+selecao[1]-selecao[0])%360; i++) {

		double diferenca = maior - menor;
		diferenca = diferenca<0?-diferenca:diferenca;
		double tamanho = 2.0/( (360+selecao[1]-selecao[0])%360);
		



		glBegin(GL_LINE_STRIP);
		



		switch (nivelColorido) {
			case COLORIDO_E_PRETO:
			case COLORIDO_E_BRANCO:
				for (unsigned int i = 0; i <= (360+selecao[1]-selecao[0])%360; i++) {
						getLineColor( (selecao[0]+i+ajusteA)%360, selecao[2]+ajusteR);
						glVertex3f(-1.0 + i*tamanho, -1.0 + (matriz.getTabela((360+selecao[0]+i+ajusteA)%360, selecao[2]+ajusteR)+25)/50.0, 0.0);
				} // for (unsigned int i = 0; i <= (360+selecao[1]-selecao[0])%360; i++)
				break;
			case PRETO_E_BRANCO:
				glColor3f(0.0,0.0,0.0);
				for (unsigned int i = 0; i <= (360+selecao[1]-selecao[0])%360; i++) {
						glVertex3f(-1.0 + i*tamanho, -1.0 + (matriz.getTabela((360+selecao[0]+i+ajusteA)%360, selecao[2]+ajusteR)+25)/50.0, 0.0);
				} // for (unsigned int i = 0; i <= (360+selecao[1]-selecao[0])%360; i++)
				break;
			case BRANCO_E_PRETO:
				glColor3f(1.0,1.0,1.0);
				for (unsigned int i = 0; i <= (360+selecao[1]-selecao[0])%360; i++) {
						glVertex3f(-1.0 + i*tamanho, -1.0 + (matriz.getTabela((360+selecao[0]+i+ajusteA)%360, selecao[2]+ajusteR)+25)/50.0, 0.0);
				} // for (unsigned int i = 0; i <= (360+selecao[1]-selecao[0])%360; i++)
				break;
			default:
				break;
		}



		glEnd();
		


	} // else
} // void displayReta(void) 








void displayDados() {
	glMatrixMode(GL_MODELVIEW);
	glShadeModel(GL_FLAT);
	







	switch (nivelColorido) {
		case PRETO_E_BRANCO:
		case COLORIDO_E_BRANCO:
			glClearColor(1.0, 1.0, 1.0, 0.0); // Fundo
			glColor3f(0.0, 0.0, 0.0);
			break;
		case BRANCO_E_PRETO:
		case COLORIDO_E_PRETO:
			glClearColor(0.0, 0.0, 0.0, 0.0); // Fundo
			glColor3f(1.0, 1.0, 1.0);
			break;
		default:
			glClearColor(1.0, 1.0, 1.0, 0.0); // Fundo
			glColor3f(0.0, 0.0, 0.0);
			break;
	}






	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);




	// 0.015625 tamanho suposto da fonte
	// Cada palavra tem 10y 28x
	static const float espacoX = 0.13125, espacoY = 0.0625;
	float tmp;
	char texto[10];




	// Limita a exibio entre os Raios que possuem valores
	if ((primeiroRaio + ajusteR) < (signed int) matriz.getRaioInicial() - 0) {
		ajusteR = matriz.getRaioInicial() - primeiroRaio;
	}
	else if ( (primeiroRaio + ajusteR) > (signed int) matriz.getRaioFinal()) {
		ajusteR = matriz.getRaioFinal() - primeiroRaio;
	}
	else if ( (primeiroRaio  + ajusteR + 29) > (signed int) matriz.getRaioFinal() - 0) {
		ajusteR = matriz.getRaioFinal() - primeiroRaio - 29;
	}




	// Imprime a escala, graus e kilometros
	for (int i = 1; i < 15; i++) { // Graus
		glRasterPos2f(-0.9515625+ i*espacoX, 0.9375);
		sprintf(texto, "%5i", (360+primeiroAngulo + i-1+ajusteA)%360 );
		drawString(texto, 0);
	}




	switch (unidadeMedidas) {
		case MILHAS:
			glRasterPos2f(-0.99, 0.9375); //-0.9671875
			drawString("Milas\\Az", 0);
			for (int j = 1; j < 31; j++) { // Raios
				glRasterPos2f(-0.99, 0.9375 - j*espacoY);
				sprintf(texto, "%6.2f Mi", (primeiroRaio + j-1+ajusteR)*0.15534275);
				drawString(texto, 0);
			}
			break;	
		case KILOMETROS:
		default:
			glRasterPos2f(-0.99, 0.9375); //-0.9671875
			drawString("Km\\Az", 0);
			for (int j = 1; j < 31; j++) { // Raios
				glRasterPos2f(-0.99, 0.9375 - j*espacoY);
				sprintf(texto, "%6.2f km", (primeiroRaio + j-1+ajusteR)/4.0);
				drawString(texto, 0);
			}
			break;	
	}








	// Imprime os valores da tabela
	for (int i = 1; i < 15; i++) {
		for (int j = 1; j < 31; j++) {
			if (nivelColorido == COLORIDO_E_BRANCO || nivelColorido == COLORIDO_E_PRETO) {
				getNewColor(1,  (360+i+primeiroAngulo-1+ajusteA)%360 , j+primeiroRaio-1+ajusteR);
			}

			glRasterPos2f(-0.9515625 + i*espacoX, 0.9375 - j*espacoY);
			tmp = matriz.getTabela( (360+i+primeiroAngulo-1+ajusteA)%360, j+primeiroRaio-1+ajusteR);
			if (tmp < -20.0) {
				drawString("--.-", 0);
			}
			else {
				sprintf(texto, "%+5.1f", tmp);
				drawString(texto, 0);
			}
		}
	} //for (int i = 0; i = 13; i++)


	glutSwapBuffers();

}











void displayMenu(void) {
	if (glutLayerGet(GLUT_OVERLAY_DAMAGED)) {
		glMatrixMode(GL_MODELVIEW);
		if (qualidadeImagem == BOA) {
			glShadeModel(GL_SMOOTH);
		}
		else {
			glShadeModel(GL_FLAT);
		}
		glClearColor(0.0, 0.0, 0.0, 0.0); // Fundo
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	} // if (glutLayerGet(GLUT_OVERLAY_DAMAGED)) 





	char tmp[100], aux[4];





	glColor3f(1.0, 1.0, 1.0);
	sprintf(tmp, matriz.getHora());
	strcat(tmp, ":");
	strcat(tmp, matriz.getMinuto());
	strcat(tmp, ":");
	strcat(tmp, matriz.getSegundo());
	glRasterPos2f(-0.55, 0.5);
	drawString(tmp, 3);




	sprintf(tmp, matriz.getDia());
	sprintf(aux, matriz.getMes());




	switch (aux[0]) {
		case '0':
			switch (aux[1]) {
				case '1':
					strcat(tmp, " JAN ");
					break;
				case '2':
					strcat(tmp, " FEV ");
					break;
				case '3':
					strcat(tmp, " MAR ");
					break;
				case '4':
					strcat(tmp, " ABR ");
					break;
				case '5':
					strcat(tmp, " MAI ");
					break;
				case '6':
					strcat(tmp, " JUN ");
					break;
				case '7':
					strcat(tmp, " JUL ");
					break;
				case '8':
					strcat(tmp, " AGO ");
					break;
				case '9':
					strcat(tmp, " SET ");
					break;
				case '0':
					strcat(tmp, " --- ");
					break;
			}
			break;
		case '1':
			switch (aux[1]) {
				case '0':
					strcat(tmp, " OUT ");
					break;
				case '1':
					strcat(tmp, " NOV ");
					break;
				case '2':
					strcat(tmp, " DEZ ");
					break;
			}
			break;
	}




	sprintf(aux, matriz.getAno());
	if (tmp[0] == '0' && tmp[1] == '0')
		strcat(tmp, "00");
	else if (aux[0] > '5')
		strcat(tmp, "19");
	else strcat(tmp, "20");
	strcat(tmp, aux);
	strcat(tmp, " BRT");




	glRasterPos2f(-0.90625, 0.45);
	drawString(tmp, 2);





	// imprime o nome da cidade
	sprintf(aux, matriz.getLocal());
	if (!strcmp(aux, "PPR"))
		sprintf(tmp, "Local: Presidente Prudente");
	else if (!strcmp(aux, "BRU"))
		sprintf(tmp, "Local: Bauru");
	else
		sprintf(tmp, "Local: Desconhecido");

	glRasterPos2f(-0.96, 0.40);
	drawString(tmp, 1);

	



	switch (qualidadeImagem) {
		case BOA:
			sprintf(tmp, "Qualidade: Maxima");
			break;
		case RUIM:
			sprintf(tmp, "Qualidade: Media");
			break;
		case PIOR:
			sprintf(tmp, "Qualidade: Minima");
			break;
	}



	glRasterPos2f(-0.95, 0.95);
	drawString(tmp, 0);




	switch (unidadeMedidas) {
		case MILHAS:
			sprintf(tmp, "Unidades: Milhas");
			break;
		case KILOMETROS:
			sprintf(tmp, "Unidades: Quilometros");
			break;
	}
	glRasterPos2f(-0.95, 0.90);
	drawString(tmp, 0);




	switch (coresSuave) {
		case COR_MAXIMO:
			sprintf(tmp, "Cores: Maximo");
			break;
		case COR_MEDIO:
			sprintf(tmp, "Cores: 32");
			break;
		case COR_MINIMO:
			sprintf(tmp, "Cores: 5");
			break;
		case COR_BRILHO:
			sprintf(tmp, "Cores: Tons de Cinza");
			break;
	}
	glRasterPos2f(-0.95, 0.85);
	drawString(tmp, 0);




	switch (qualidadeR) {
		case 1:
			sprintf(tmp, "Tamanho da Leitura: 250 m");
			break;
		case 2:
			sprintf(tmp, "Tamanho da Leitura: 500 m");
			break;
		case 4:
			sprintf(tmp, "Tamanho da Leitura: 1 km");
			break;
	}
	glRasterPos2f(-0.95, 0.80);
	drawString(tmp, 0);




	drawCores();



	glutSwapBuffers();
}











/*******************************************************************************
 *******************************************************************************
 *******************************************************************************
                 Funes responsveis pelo controle do Mouse
 *******************************************************************************
 *******************************************************************************
 *******************************************************************************/











void mouse(int button, int state, int x, int y) {

	/* Aqui, basicamente leio os X e Y do mouse, e transformo eles em coordenadas -1,+1 reais */
	mXI = (width - height)/2;
	mYI = (height - width)/2;
	float kx, ky, ax, ay, wx, wy;

	if(button != GLUT_LEFT_BUTTON) {
		return;
	}




	
	if(width > height) {
		kx = width - mXI*2;
		ax = x - mXI;
		
		wx = (2.0 * ax) /(kx - 1) - 1.0;
		wy = (2.0 *(height - 1 - y)) /(float)(height - 1) - 1.0;
	} // if(width > height)
	else {
		ky = height - mYI*2;
		ay = y - mYI;
		wx = (2.0 * x) /(width - 1) - 1.0;
		wy = (2.0 *(ky - 1 - ay)) /(float)(ky - 1) - 1.0;
	} // else





	/* Se eu obtiver coordenadas impossveis, ignoro o clique do mouse */

	if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0)) {
		return;
	} // if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0)) 






	/* Seleo de estados. Serve para ver qual ao o mouse vai tomar, baseado em qual
	 *	 o estado atual do programa (variavel estado) e qual era o estado anterior.
	 *	A partir disso, ele escolhe o que fazer. */

	switch(estado) {
		case SELECIONA_FATIA:
			/* Le apenas cliques esquerdos nessa fase do programa */
			if(button != GLUT_LEFT_BUTTON) {
				return;
				}

			/* Seto o tipo de figura para desenhar na seleo como sendo fatia */
			figure.kind = 2;

			/* Esse bloco de funo basicamente le o boto sendo apertado e sendo solto */
			if (button == GLUT_LEFT_BUTTON) {
				if (state == GLUT_DOWN)
					selecaoInicia(wx, wy);
				if (state == GLUT_UP)
					selecaoTermina(wx, wy);
			}





			/* NOTA: ContaPontos fica sem utilidade com o novo mtodo. */
			/* Organizamos os pontos para que o intervalo no seja infinito, e iniciamos o novo estado  */
			if(contaPontos == 2) {
				pontos[0][0] = (360 + Aprox(atan2(figure.y0, figure.x0)*180 / PI))%360;
				pontos[0][1] = Aprox(RAIO*sqrt( figure.x0*figure.x0 + figure.y0*figure.y0))-4;
				pontos[1][0] = (360 + Aprox(atan2(figure.y1, figure.x1)*180 / PI))%360;
				pontos[1][1] = Aprox(RAIO*sqrt( figure.x1*figure.x1 + figure.y1*figure.y1))-4;



				resetFigure();
				ordenaVetor();



				/* Calcula-se */
				for (unsigned int w = 0; w <= abertura; w++) {
					vetorDados();
				} // for(unsigned int w = 0; w <= abertura; w++)



				estado = EXIBE_FATIA;
				contaPontos = 0;
			} // if(contaPontos == 2) 



			break;
		case SELECIONA_CORTE_DE_ZOOM:
			// Apenas Cliques esquerdos!
			if(button != GLUT_LEFT_BUTTON) {
					return;
			}




			/* Seto o tipo de figura para desenhar na seleo como sendo fatia */
			figure.kind = 2;

			/* Esse bloco de funo basicamente le o boto sendo apertado e sendo solto */
			if (button == GLUT_LEFT_BUTTON) {
				if (state == GLUT_DOWN)
					selecaoInicia(wx, wy);
				if (state == GLUT_UP)
					selecaoTermina(wx, wy);
			}




			/* NOTA: ContaPontos fica sem utilidade com o novo mtodo. */
			/* Organizamos os pontos para que o intervalo no seja infinito, e iniciamos o novo estado  */
			if(contaPontos == 2) {
				figure.x0 = figure.x0/escalaGlobal + xZero;
				figure.x1 = figure.x1/escalaGlobal + xZero;
				figure.y0 = figure.y0/escalaGlobal + yZero;
				figure.y1 = figure.y1/escalaGlobal + yZero;
				pontos[0][0] = (360 + Aprox(atan2(figure.y0, figure.x0)*180 / PI))%360;
				pontos[0][1] = Aprox(RAIO*sqrt( figure.x0*figure.x0 + figure.y0*figure.y0))-4;
				pontos[1][0] = (360 + Aprox(atan2(figure.y1, figure.x1)*180 / PI))%360;
				pontos[1][1] = Aprox(RAIO*sqrt( figure.x1*figure.x1 + figure.y1*figure.y1))-4;
				



				ordenaVetor();
				resetFigure();




				/* Calcula-se */
				for (unsigned int w = 0; w <= abertura; w++) {
					vetorDados();
				} // for(unsigned int w = 0; w <= abertura; w++)

				estado = EXIBE_FATIA;
				contaPontos = 0;
			} // if(contaPontos == 2) 



			break;


		case SELECIONA_ZOOM_DE_CORTE:
			
			// Apenas Cliques esquerdos!
			if(button != GLUT_LEFT_BUTTON) {
					return;
			} // if(button != GLUT_LEFT_BUTTON || state != GLUT_DOWN)
			


			figure.kind = 1;




			/* Esse bloco de funo basicamente le o boto sendo apertado e sendo solto */
			if (button == GLUT_LEFT_BUTTON) {
				if (state == GLUT_DOWN)
					selecaoInicia(wx, wy);
				if (state == GLUT_UP)
					selecaoTermina(wx, wy);
			}






			if (contaPontos == 2) { // aqui!
				float aux, auy;
				if (figure.x1 < figure.x0) {
					aux = figure.x1;
					figure.x1 = figure.x0;
					figure.x0 = aux;
					}
				if (figure.y1 < figure.y0) {
					auy = figure.y1;
					figure.y1 = figure.y0;
					figure.y0 = auy;
					}
				figure.x0 = figure.x0/escalaGlobal + xZero;
				figure.x1 = figure.x1/escalaGlobal + xZero;
				figure.y0 = figure.y0/escalaGlobal + yZero;
				figure.y1 = figure.y1/escalaGlobal + yZero;
				quadradoZoom[0] = figure.x0;
				quadradoZoom[1] = figure.y0;
				aux = figure.x1 - figure.x0;
				auy = figure.y1 - figure.y0;





				if (! (aux > 0 && auy > 0)) {
					contaPontos = 0;
					return;
				} // if (! (aux > 0 && auy > ) {




				aux = aux > auy ? aux : auy;
				



				resetFigure();
				zoomSimples(aux, quadradoZoom[0], quadradoZoom[1]);
				estado = EXIBE_FATIA;
				contaPontos = 0;
			}
			break;
		case SELECIONA_ZOOM:



			// Apenas Cliques esquerdos!
			if(button != GLUT_LEFT_BUTTON) {
					return;
			} // if(button != GLUT_LEFT_BUTTON || state != GLUT_DOWN)
			



			figure.kind = 1;





			/* Esse bloco de funo basicamente le o boto sendo apertado e sendo solto */
			if (button == GLUT_LEFT_BUTTON) {
				if (state == GLUT_DOWN)
					selecaoInicia(wx, wy);
				if (state == GLUT_UP)
					selecaoTermina(wx, wy);
			}




			if (contaPontos == 2) { // aqui!
				float aux, auy;
				if (figure.x1 < figure.x0) {
					aux = figure.x1;
					figure.x1 = figure.x0;
					figure.x0 = aux;
					}
				if (figure.y1 < figure.y0) {
					auy = figure.y1;
					figure.y1 = figure.y0;
					figure.y0 = auy;
					}
				figure.x0 = figure.x0/escalaGlobal + xZero;
				figure.x1 = figure.x1/escalaGlobal + xZero;
				figure.y0 = figure.y0/escalaGlobal + yZero;
				figure.y1 = figure.y1/escalaGlobal + yZero;

				quadradoZoom[0] = figure.x0;
				quadradoZoom[1] = figure.y0;
				aux = figure.x1 - figure.x0;
				auy = figure.y1 - figure.y0;

				if (! (aux > 0 && auy > 0)) {
					contaPontos = 0;
					return;
				}
				resetFigure();
				zoomSimples(aux, quadradoZoom[0], quadradoZoom[1]);
				estado = EXIBE_ZOOM;
				contaPontos = 0;
			}
			break;
		case SELECIONA_GRAFICO: // editando agora
			if (button != GLUT_LEFT_BUTTON) {
				return;
			} // if(state != GLUT_DOWN && state != GLUT_LEFT_BUTTON)
			else {




				GLfloat wx, wy;
				
				if(width > height) {
					kx = width - mXI*2;
					ax = x - mXI;
					



					wx = (2.0 * ax)/(kx - 1) - 1.0;
					wy = (2.0 *(height - 1 - y))/(float)(height - 1) - 1.0;
				} // if(width > height) 
				else {
					ky = height - mYI*2;
					ay = y - mYI;
					wx = (2.0 * x)/(width - 1) - 1.0;
					wy = (2.0 *(ky - 1 - ay))/(float)(ky - 1) - 1.0;
				} // else




				if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0)) {
					return;
				} // if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0)) 
				


				figure.kind = 3; // desenha arco ou reta



				if (button == GLUT_LEFT_BUTTON) {
					if (state == GLUT_DOWN)
						selecaoInicia(wx, wy);
					if (state == GLUT_UP)
						selecaoTermina(wx, wy);
				}




				// Aqui, tenho todos os valores devidamente convertidos.
				if(contaPontosReta == 2) {
					retaPontos[0][0] = azToPol((360 + Aprox(atan2(figure.y0, figure.x0)*180 / PI))%360);
					retaPontos[0][1] = Aprox(RAIO*sqrt( figure.x0*figure.x0 + figure.y0*figure.y0));
					retaPontos[1][0] = azToPol((360 + Aprox(atan2(figure.y1, figure.x1)*180 / PI))%360);
					retaPontos[1][1] = Aprox(RAIO*sqrt( figure.x1*figure.x1 + figure.y1*figure.y1));
					



					ordenaVetorReta();
					estado = EXIBE_GRAFICO;
					resetFigure();
					contaPontosReta = 0;
				} // if(contaPontosReta == 2) 
			} // else
			break;
		case SELECIONA_CENTRO:
			// Apenas Cliques esquerdos!
			if(button != GLUT_LEFT_BUTTON || state != GLUT_DOWN) {
					return;
			} // if(button != GLUT_LEFT_BUTTON || state != GLUT_DOWN)
			



			if(width > height) {
				kx = width - mXI*2;
				ax = x - mXI;
				


				wx = (2.0 * ax) /(kx - 1) - 1.0;
				wy = (2.0 *(height - 1 - y)) /(float)(height - 1) - 1.0;
			} // if(width > height)
			else {
				ky = height - mYI*2;
				ay = y - mYI;
				wx = (2.0 * x) /(width - 1) - 1.0;
				wy = (2.0 *(ky - 1 - ay)) /(float)(ky - 1) - 1.0;
			} // else




			if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0)) {
				estado = estadoAnterior;
				return;
			} // if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0)) 
			



			wx = wx/escalaGlobal + xZero;
			wy = wy/escalaGlobal + yZero;




			xZero = wx;
			yZero = wy;




			estado = estadoAnterior;
			break;
		case SELECIONA_NUMEROS:



			// Apenas Cliques esquerdos!
			if(button != GLUT_LEFT_BUTTON || state != GLUT_DOWN) {
					return;
			} // if(button != GLUT_LEFT_BUTTON || state != GLUT_DOWN)
			



			if(width > height) {
				kx = width - mXI*2;
				ax = x - mXI;
				



				wx = (2.0 * ax) /(kx - 1) - 1.0;
				wy = (2.0 *(height - 1 - y)) /(float)(height - 1) - 1.0;
			} // if(width > height)
			else {
				ky = height - mYI*2;
				ay = y - mYI;
				wx = (2.0 * x) /(width - 1) - 1.0;
				wy = (2.0 *(ky - 1 - ay)) /(float)(ky - 1) - 1.0;
			} // else





			if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0)) {
				return;
			} // if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0)) 
			




			switch (estadoAnterior) {
				case SELECIONA_FATIA:
				case ESPERA:
					break;
				default:
					wx = wx/escalaGlobal + xZero;
					wy = wy/escalaGlobal + yZero;
					break;
			}
			primeiroRaio = Aprox(RAIO*sqrt( wx*wx + wy*wy)) - 15;
			primeiroAngulo = azToPol((360 + Aprox(atan2(wy, wx)*180 / PI) + 6 )%360);
			estado = EXIBE_NUMEROS;
			glutSwapBuffers();
			glutPostRedisplay();
			break;
		//case ESPERA:
		case ESPERA:
		case EXIBE_NUMEROS:
		case EXIBE_GRAFICO:
		case EXIBE_ZOOM:
		case EXIBE_FATIA:
			if(button != GLUT_LEFT_BUTTON || state != GLUT_DOWN) {
					return;
			} // if(button != GLUT_LEFT_BUTTON || state != GLUT_DOWN)
			teclado(27,0,0);
			break;
		default:
			escalaGlobal = 1.0;
			xZero = 0.0;
			yZero = 0.0;
			estado = ESPERA;
			break;
	} // switch(estado)




	return;
} // void mouse(int button, int state, int x, int y)









void mousePosicao(int x, int y) {
	mXI = (width - height)/2;
	mYI = (height - width)/2;
	GLfloat wx, wy, kx, ky, ax, ay;
	unsigned int rtmp;
	rtmp = width > height ? height : width;





	if(width > height) {
		kx = width - mXI*2;
		ax = x - mXI;



		wx = (2.0 * ax) /(kx - 1) - 1.0;
		wy = (2.0 *(height - 1 - y)) /(float)(height - 1) - 1.0;
	} // if(width > height) 
	else {
		ky = height - mYI*2;
		ay = y - mYI;
		wx = (2.0 * x) /(width - 1) - 1.0;
		wy = (2.0 *(ky - 1 - ay)) /(float)(ky - 1) - 1.0;
	} // else




	//if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0)) {
	//	return;
	//} // if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0))

	switch (estado) {
		case ESPERA:
		case SELECIONA_FATIA:
			// Texto
			


			rtmp = Aprox(RAIO*sqrt( wx*wx + wy*wy));
			Graio = RAIO*sqrt( wx*wx + wy*wy)/4.0;
			Gazimute = azToPol((360 + Aprox(atan2(wy, wx)*180 / PI))%360);
			Gintensidade = matriz.getTabela(Gazimute, rtmp);
			break;
		case EXIBE_FATIA:
		case SELECIONA_ZOOM_DE_CORTE:
		case SELECIONA_GRAFICO:
		case EXIBE_ZOOM:
		case SELECIONA_NUMEROS:
		case SELECIONA_ZOOM:
		case SELECIONA_CENTRO:




			wx = wx/escalaGlobal + xZero;
			wy = wy/escalaGlobal + yZero;
			// Aqui, tenho todos os valores devidamente convertidos.




			rtmp = Aprox(RAIO*sqrt( wx*wx + wy*wy));
			Graio = RAIO*sqrt( wx*wx + wy*wy)/4.0;
			Gazimute = azToPol((360 + Aprox(atan2(wy, wx)*180 / PI))%360);
			Gintensidade = matriz.getTabela(Gazimute, rtmp);
			break;
		case EXIBE_GRAFICO:
			wx = (wx + 1.0)/2.0;
			wy = (wy + 1.0)/2.0;
			unsigned int distancia;




			if (isLinha == true) {
				distancia = selecao[1]-selecao[0];
				rtmp = Aprox(selecao[0]+ajusteR + distancia*wx);
				Graio = (selecao[0] + ajusteR + distancia*wx)/4.0; // por que 4?
				Gazimute = (selecao[2]+ajusteA)%360;
			} // if (isLinha == true)
			else {
				distancia = (360 + selecao[1]-selecao[0])%360;
				rtmp = selecao[2]+ajusteR;
				Graio = (selecao[2]+ajusteR)/4.0;
				Gazimute = (selecao[0]+ajusteA+Aprox(wx*distancia/2.0))%360;
			} // else 



			Gintensidade = matriz.getTabela(Gazimute, rtmp);
			break;
		case EXIBE_NUMEROS:
			return;
		default:
			estado = estadoAnterior;
	} // switch (estado) 
	glutSetWindow(janelaPrincipal);
	glutPostRedisplay();
	glutSetWindow(janelaMenu);
	glutPostRedisplay();
	/*
	arrumar isso pra soltar os dados s na outra janela
	*/



} // void mousePosicao(int x, int y)









void mouseMovimento(int x, int y) {
	int mXI = (width - height)/2;
	int mYI = (height - width)/2;
	float kx, ky, ax, ay, wx, wy;
	unsigned int rtmp;
	rtmp = width > height ? height : width;




	if(width > height) {
		kx = width - mXI*2;
		ax = x - mXI;
		
		wx = (2.0 * ax) /(kx - 1) - 1.0;
		wy = (2.0 *(height - 1 - y)) /(float)(height - 1) - 1.0;
	}




	else {
		ky = height - mYI*2;
		ay = y - mYI;
		wx = (2.0 * x) /(width - 1) - 1.0;
		wy = (2.0 *(ky - 1 - ay)) /(float)(ky - 1) - 1.0;
	}
	if( !(wx <= 1.0 && wx >= -1.0 && wy <= 1.0 && wy >= -1.0)) {
		return;
	}
	xSelecaoAtual = wx;
	ySelecaoAtual = wy;





	switch (estado) {
		case ESPERA:
		case SELECIONA_FATIA:
			// Texto
			



			rtmp = Aprox(RAIO*sqrt( wx*wx + wy*wy));
			Graio = RAIO*sqrt( wx*wx + wy*wy)/4.0;
			Gazimute = azToPol((360 + Aprox(atan2(wy, wx)*180 / PI))%360);
			Gintensidade = matriz.getTabela(Gazimute, rtmp);
			break;
		case EXIBE_FATIA:
		case SELECIONA_GRAFICO:
		case SELECIONA_ZOOM_DE_CORTE:
		case EXIBE_ZOOM:
		case SELECIONA_NUMEROS:
		case SELECIONA_ZOOM:
		case SELECIONA_CENTRO:




			wx = wx/escalaGlobal + xZero;
			wy = wy/escalaGlobal + yZero;
			// Aqui, tenho todos os valores devidamente convertidos.



			rtmp = Aprox(RAIO*sqrt( wx*wx + wy*wy));
			Graio = RAIO*sqrt( wx*wx + wy*wy)/4.0;
			Gazimute = azToPol((360 + Aprox(atan2(wy, wx)*180 / PI))%360);
			Gintensidade = matriz.getTabela(Gazimute, rtmp);
			break;
		case EXIBE_GRAFICO:
			wx = (wx + 1.0)/2.0;
			wy = (wy + 1.0)/2.0;
			unsigned int distancia;




			if (isLinha == true) {
				distancia = selecao[1]-selecao[0];
				rtmp = Aprox(selecao[0]+ajusteR + distancia*wx);
				Graio = (selecao[0] + ajusteR + distancia*wx)/4.0; // por que 4?
				Gazimute = (selecao[2]+ajusteA)%360;
			} // if (isLinha == true)
			else {
				distancia = (360 + selecao[1]-selecao[0])%360;
				rtmp = selecao[2]+ajusteR;
				Graio = (selecao[2]+ajusteR)/4.0;
				Gazimute = (selecao[0]+ajusteA+Aprox(wx*distancia/2.0))%360;
			} // else 



			Gintensidade = matriz.getTabela(Gazimute, rtmp);
			break;
		case EXIBE_NUMEROS:
			return;
		default:
			estado = estadoAnterior;
	} // switch (estado) 
	


	glutPostRedisplay();
}








/*******************************************************************************
 *******************************************************************************
 *******************************************************************************
                        Funes responsveis pelos Menus
 *******************************************************************************
 *******************************************************************************
 *******************************************************************************/








void menuMain(int valor) {
	switch (valor) {
		case 1: // Ampliar
			switch (estado) {
				case ESPERA:
				case SELECIONA_FATIA:
				case EXIBE_ZOOM:
					estado = SELECIONA_ZOOM;
					contaPontos = 0;
					break;
				case EXIBE_FATIA:
					estado = SELECIONA_ZOOM_DE_CORTE;
					contaPontos = 0;
					break;
			}
			break;
			// adicionar opo de salvar cortes
			// matriz.salvaAtual(pontos[0][1], pontos[1][1], pontos[0][0], pontos[1][0]);
		// > submenu de opes
		case 2: // Voltar
			switch (estado) {
				case EXIBE_GRAFICO:
					switch (estadoAnterior) {
						case EXIBE_FATIA:
						case SELECIONA_ZOOM_DE_CORTE:
						case SELECIONA_GRAFICO:
							estado = EXIBE_FATIA;
							break;
						case EXIBE_ZOOM:
						case SELECIONA_ZOOM:
						case SELECIONA_CORTE_DE_ZOOM:
							estado = ESPERA;
							escalaGlobal = 1.0;
							xZero = 0.0;
							yZero = 0.0;
							break;
						case ESPERA: // volta para o estado anterior
							estado = estadoAnterior;
							escalaGlobal = escalaGlobalOld;
							xZero = xZeroOld;
							yZero = yZeroOld;
							



							estadoAnterior = ESPERA;
							escalaGlobalOld = 1.0;
							xZeroOld = 0.0;
							yZeroOld = 0.0;
							break;
						default:
							estadoAnterior = EXIBE_GRAFICO;
							estado = ESPERA;
							glutPostRedisplay();
							break;
					}
				case ESPERA: // volta para o estado anterior
					estado = estadoAnterior;
					escalaGlobal = escalaGlobalOld;
					xZero = xZeroOld;
					yZero = yZeroOld;
					



					estadoAnterior = ESPERA;
					escalaGlobalOld = 1.0;
					xZeroOld = 0.0;
					yZeroOld = 0.0;
					break;
				default: // todo menu voltar xxxxxxx
					estadoAnterior = estado;
					escalaGlobalOld = escalaGlobal;
					xZeroOld = xZero;
					yZeroOld = yZero;
					



					estado = ESPERA;
					escalaGlobal = 1.0;
					xZero = 0.0;
					yZero = 0.0;
					break;
			}
			break;
		case 3:
			exit(0);
			break;
	} // switch (valor)
} // void menuMain(int valor)











void menuSubGrafico(int valor) {
	switch (valor) {
		case 1:
			estadoAnterior = estado;
			estado = SELECIONA_GRAFICO;
			contaPontosReta = 0;
			ajusteA = 0;
			ajusteR = 0;
			isLinha = false;
			break;
		case 2:
			estadoAnterior = estado;
			estado = SELECIONA_GRAFICO;
			contaPontosReta = 0;
			ajusteA = 0;
			ajusteR = 0;
			isLinha = true;
			break;
	} // switch (valor) 

} // void menuSubGrafico(int valor)











void menuSubCorQuantidade(int valor) {
	switch (valor) {
		case 1:
			coresSuave = COR_MAXIMO;
			break;
		case 2:
			coresSuave = COR_MEDIO;
			break;
		case 3:
			coresSuave = COR_MINIMO;
			break;
		case 4:
			coresSuave = COR_BRILHO;
			break;
	} // switch (valor) 
	glutSetWindow(janelaMenu);
	glutSwapBuffers();
	glutPostRedisplay();
	glutSetWindow(janelaPrincipal);
	glutSwapBuffers();
	glutPostRedisplay();
} //











void menuSubGraficoSelect(int valor) {
	// chama o submenu glutAddSubMenu("Grafico Escalar", idMenuSubGrafico);
	switch (valor) {
		case 1: // Seleciona Fatia
			estado = SELECIONA_FATIA;
			contaPontos = 0;
			break;
		case 2: // Altera ponto central
			switch (estado) {
				case EXIBE_ZOOM:
				case SELECIONA_ZOOM:
				case EXIBE_FATIA:
				case SELECIONA_GRAFICO:
				case SELECIONA_CENTRO:
					estadoAnterior = estado;
					estado = SELECIONA_CENTRO;
					break;
				default:
					break;
			}
			break;


		case 3: // Mostra tabela de Valores
			switch (estado) {
				case ESPERA:
				case SELECIONA_FATIA:
				case EXIBE_ZOOM:
				case SELECIONA_ZOOM:
				case SELECIONA_ZOOM_DE_CORTE:
				case EXIBE_FATIA:
				case SELECIONA_GRAFICO:
				case SELECIONA_CORTE_DE_ZOOM:
				case SELECIONA_CENTRO:
					ajusteA = 0;
					ajusteR = 0;
					estadoAnterior = estado;
					estado = SELECIONA_NUMEROS;
					break;
				case EXIBE_GRAFICO:
					ajusteA = 0;
					ajusteR = 0;
					primeiroRaio = Aprox(Aprox(Graio*4.0)) - 15;
					primeiroAngulo = Gazimute;
					estadoAnterior = EXIBE_GRAFICO;
					estado = EXIBE_NUMEROS;
					glutSwapBuffers();
					glutPostRedisplay();

					break;
				default:
					estadoAnterior = ESPERA;
					estado = ESPERA;
					escalaGlobal = 1.0;
					xZero = 0.0;
					yZero = 0.0;
					break;
			}
			break;



		default:
			break;
	} // switch (valor) 
} // void menuSubGrafico(int valor)










void menuDados(int valor) {
	valor = 0;
} // void menuDados(int valor)








void menuSubDadosCores(int valor) {
	switch (valor) {
		case 1:
			nivelColorido = PRETO_E_BRANCO;
			break;
		case 2:
			nivelColorido = BRANCO_E_PRETO;
			break;
		case 3:
			nivelColorido = COLORIDO_E_PRETO;
			break;
		case 4:
			nivelColorido = COLORIDO_E_BRANCO;
			break;
		default:
			break;
	} // switch (valor) 





	glutSetWindow(janelaMenu);
	glutSwapBuffers();
	glutPostRedisplay();
	glutSetWindow(janelaPrincipal);
	glutSwapBuffers();
	glutPostRedisplay();
} // void menuSubDadosCores(int valor)











/*void menuSubDadosVoltar(int valor) {
	switch (valor) {
		case 1:
			estado = estadoAnterior;
			break;
		case 2: // "Tela Anterior nestas coordenadas"
			
			break;
		default:
			break;
	} // switch (valor) 
	glutSwapBuffers();
	glutPostRedisplay();
} // void menuSubDadosVoltar(int valor)
*/












void menuSubMedidas(int valor) {
	switch (valor) {
		case 1: // "Kilometros"
			unidadeMedidas = KILOMETROS;
			break;
		case 2: // "Milhas"
			unidadeMedidas = MILHAS;
			break;
		default:
			unidadeMedidas = KILOMETROS;
			break;
	} // switch (valor) 
	glutSetWindow(janelaPrincipal);
	glutSwapBuffers();
	glutPostRedisplay();
	glutSetWindow(janelaMenu);
	glutSwapBuffers();
	glutPostRedisplay();
} // void menuSubMedidas(int valor)












void menuSubCores(int valor) {
	switch (valor) {
		case 1:
			qualidadeImagem = BOA;
			break;
		case 2:
			qualidadeImagem = RUIM;
			break;
		case 3:
			qualidadeImagem = PIOR;
			break;
		default:
			break;
	} // switch (valor) 







	glutSetWindow(janelaMenu);
	glutSwapBuffers();
	glutPostRedisplay();
	glutSetWindow(janelaPrincipal);
	glutSwapBuffers();
	glutPostRedisplay();
} // void menuSubCores(int valor)













void menuTiff(int value) {
	switch (value) {
		case 1:
			writetiff(imageName(), "Arquivo gerado pelo OpenGL", 0, 0, width, height, COMPRESSION_LZW);
			break;
		case 2:
			writetiff(imageName(), "Arquivo gerado pelo OpenGL", 0, 0, width, height, COMPRESSION_PACKBITS);
			break;
		case 3:
			writetiff(imageName(), "Arquivo gerado pelo OpenGL", 0, 0, width, height, COMPRESSION_NONE);
			break;
	}
}








void menuSalvar(int value) {
	switch (value) {
		case 1:
			switch (estado) {
				case EXIBE_FATIA:
					matriz.salvaAtual(pontos[0][1], pontos[1][1], pontos[0][0], pontos[1][0]);
					break;
				case EXIBE_GRAFICO:
					if (isLinha == true)
						matriz.salvaAtual(selecao[0]+ajusteR, selecao[1]+ajusteR, azToPol((360+selecao[2]+ajusteA)%360), azToPol((360+selecao[2]+ajusteA)%360));
					else
						matriz.salvaAtual(selecao[2]+ajusteR, selecao[2]+ajusteR, azToPol((360+selecao[0]+ajusteA)%360), azToPol((360+selecao[1]+ajusteA)%360));
					break;
				case EXIBE_NUMEROS:
					matriz.salvaAtual((primeiroRaio + ajusteR), (primeiroRaio + 30 + ajusteR), azToPol((360+primeiroAngulo + ajusteA)%360), azToPol((360+primeiroAngulo + 13 + ajusteA)%360));
					break;
				}
			break;
	}
}











void menuSubLeituraQualidade(int value) {
	switch (value) {
		case 1:
			qualidadeR = 1;
			break;
		case 2:
			qualidadeR = 2;
			break;
		case 3:
			qualidadeR = 4;
			break;
	}







	glutSetWindow(janelaMenu);
	glutSwapBuffers();
	glutPostRedisplay();
	glutSetWindow(janelaPrincipal);
	glutSwapBuffers();
	glutPostRedisplay();
}















/*******************************************************************************
 *******************************************************************************
 *******************************************************************************
            Funes responsveis por desenhar objetos para display
 *******************************************************************************
 *******************************************************************************
 *******************************************************************************/










void drawPosition() {
	char teste[100], tmpteste[100];
	glColor3f(1.0F, 1.0F, 1.0F);






	switch (unidadeMedidas) {
		case MILHAS:
			if (Gintensidade < -25.0) {
				sprintf(teste, "Raio %5.1f Mi, Azimute %i graus, ", Graio*0.621371192, Gazimute);
			} // if (Gintensidade < -25.0)
			else {
				sprintf(teste, "Raio %5.1f Mi, Azimute %i graus, ", Graio*0.621371192, Gazimute);
			} // else
			break;
		case KILOMETROS:
		default:
			if (Gintensidade < -25.0) {
				sprintf(teste, "Raio %5.1f km, Azimute %03i graus, ", Graio, Gazimute);
			} // if (Gintensidade < -25.0)
			else {
				sprintf(teste, "Raio %5.1f km, Azimute %03i graus, ", Graio, Gazimute);
			} // else
			break;
	}
	








	//glRasterPos2f(-0.99, -0.95);
	//drawString(teste, 0);
	
	if (Gintensidade < -25.0) {
		sprintf(tmpteste, "Refletividade --.- dBZ");
	} // if (Gintensidade < -25.0)
	else {
		sprintf(tmpteste, "Refletividade %2.1f dBZ", Gintensidade);
	} // else





	strcat(teste, tmpteste);







	glRasterPos2f(-0.99, -0.99);
	drawString(teste, 0);
	//glutSwapBuffers();
	//glutPostRedisplay();
} // void drawPosition() 














void drawCidades() {
	glColor3f(0.0F, 0.0F, 0.0F);




	for (int i = 0; i < CIDADES; i++) {
		glRasterPos2f(cidade[i].x, cidade[i].y);
		drawString(cidade[i].nome, 0);
	} // for (int i = 0; i < CIDADES; i++) 
} // void drawCidades()















void drawQuadrado(int sizeR, int i, int r) {
	glBegin(GL_POLYGON); // GL_POINTS);
		int t = azToPol(i); // transforma t no angulo polar de i
		getNewColor(sizeR, i, r); // recebe a cor da posio [i][r] da matriz
		glVertex3f(eixoX(t, r*sizeR), eixoY(t, r*sizeR), 0.0f); // vertice em t,r



		int k = i;
		i = (i+1)%360;
		t = azToPol(i);
		getNewColor(sizeR, i, r); 
		glVertex3f(eixoX(t, r*sizeR), eixoY(t, r*sizeR), 0.0f); // vertice em t+1, r



		// t++;
		r = r + 1;
		getNewColor(sizeR, i, r); 
		glVertex3f(eixoX(t, r*sizeR), eixoY(t, r*sizeR), 0.0f); // vertice em t+1, r+1



		i = k;
		t = azToPol(i);
		getNewColor(sizeR, i, r); 
		glVertex3f(eixoX(t, r*sizeR), eixoY(t, r*sizeR), 0.0f); // vertice em t, r+1
	glEnd();
} // void drawQuadrado(int i, int r) 














void drawQuadradoSimples(int sizeR, int t, int r) {
	glBegin(GL_POLYGON); // GL_POINTS); 
		getNewColor(sizeR, t, r); // recebe a cor da posio [i][r] da matriz
		r *= sizeR;
		glVertex3f(eixoX(azToPol(t)-0.5, r), eixoY(azToPol(t)-0.5, r), 0.0f); // vertice em t,r
		glVertex3f(eixoX(azToPol(t)+0.5, r), eixoY(azToPol(t)+0.5, r), 0.0f); // vertice em t+1, r
		glVertex3f(eixoX(azToPol(t)+0.5, r+sizeR), eixoY(azToPol(t)+0.5, r+sizeR), 0.0f);  // vertice em t+1, r+1
		glVertex3f(eixoX(azToPol(t)-0.5, r+sizeR), eixoY(azToPol(t)-0.5, r+sizeR), 0.0f);  // vertice em t, r+1
	glEnd();
} // void drawQuadrado(int i, int r) 














void drawString(char *s, int tamanho) {
	switch (tamanho) {
		case 0:
			for(unsigned int i = 0; i < strlen(s); i++) {
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, s[i]); 
			} // for(i = 0; i < strlen(s); i++) 
			break;
		case 1:
			for(unsigned int i = 0; i < strlen(s); i++) {
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, s[i]); 
			} // for(i = 0; i < strlen(s); i++) 
			break;
		case 2:
			for(unsigned int i = 0; i < strlen(s); i++) {
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, s[i]); 
			} // for(i = 0; i < strlen(s); i++) 
			break;
		case 3:
			for(unsigned int i = 0; i < strlen(s); i++) {
				glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24, s[i]); 
			} // for(i = 0; i < strlen(s); i++) 
			break;
		default:
			for(unsigned int i = 0; i < strlen(s); i++) {
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, s[i]); 
			} // for(i = 0; i < strlen(s); i++) 
			break;
	}
} // void drawString(char *s)












void drawEscala() {
	glLineWidth(1.0);
	glColor3f(0.5, 0.5, 0.5);
	glBegin(GL_LINES);
	




	// desenha as retas saindo do centro
	//for (unsigned int r = 0; r < 2820; r = r + 20) {
		for (unsigned int t = 0; t < 180; t = t + 30) { // 360
			glVertex3f(eixoX(t, 3000), eixoY(t, 3000), 0.0);
			glVertex3f(eixoX(t+180, 3000), eixoY(t+180, 3000), 0.0);
			// glVertex3f(eixoX(t, r), eixoY(t, r), 0.0);
		} // for (unsigned int t = 0; t<360; t=t+30)
	//} // for (unsigned int r = 0; r < 2820; r=r+20)
	glEnd();

	



	int w = matriz.getRaioInicial();
	glColor3f(0.65, 0.65, 0.65);



	glBegin(GL_LINE_LOOP);
		for (unsigned int t = 0; t <360; t++) {
				glVertex3f(eixoX(t, w), eixoY(t, w), 0.0);
		} // for (unsigned int t = 0; t <360; t++)
	glEnd();
	w = matriz.getRaioFinal();
	glBegin(GL_LINE_LOOP);
		for (unsigned int t = 0; t <360; t++) {
				glVertex3f(eixoX(t, w), eixoY(t, w), 0.0);
		} // for (unsigned int t = 0; t <360; t++)
	glEnd();




	switch (unidadeMedidas) {
		case MILHAS:
			glColor3f(0.5, 0.5, 0.5);
			// desenha os circulos marcando dada distncia
			for (unsigned r = 1; r < 4; r++) {
				glBegin(GL_LINE_LOOP);
				for (unsigned int t = 0; t <360; t++) {
						glVertex3f(eixoX(t, Aprox(r*643.7376)), eixoY(t, Aprox(r*643.7376)), 0.0);
				} // for (unsigned int t = 0; t <360; t++)
				glEnd();
			} // for (unsigned r = 500; r <= 2000; r = r+500) 

			glColor3f(0.25, 0.25, 0.25);
			glRasterPos2f(eixoX(30, Aprox(643.7376)), eixoY(30, Aprox(643.7376)));
			drawString(". 100 Milhas", 0);
			glRasterPos2f(eixoX(150, Aprox(643.7376)), eixoY(150, Aprox(643.7376)));
			drawString(". 100 Milhas", 0);
			glRasterPos2f(eixoX(210, Aprox(643.7376)), eixoY(210, Aprox(643.7376)));
			drawString(". 100 Milhas", 0);
			glRasterPos2f(eixoX(330, Aprox(643.7376)), eixoY(330, Aprox(643.7376)));
			drawString(". 100 Milhas", 0);
			glRasterPos2f(eixoX(60, Aprox(2*643.7376)), eixoY(60, Aprox(2*643.7376)));
			drawString(". 200 milhas", 0);
			glRasterPos2f(eixoX(120, Aprox(2*643.7376)), eixoY(120, Aprox(2*643.7376)));
			drawString(". 200 milhas", 0);
			glRasterPos2f(eixoX(240, Aprox(2*643.7376)), eixoY(240, Aprox(2*643.7376)));
			drawString(". 200 milhas", 0);
			glRasterPos2f(eixoX(300, Aprox(2*643.7376)), eixoY(300, Aprox(2*643.7376)));
			drawString(". 200 milhas", 0);
			glRasterPos2f(eixoX(0, Aprox(3*643.7376)), eixoY(0, Aprox(3*643.7376)));
			drawString(". 300 milhas", 0);
			glRasterPos2f(eixoX(90, Aprox(3*643.7376)), eixoY(90, Aprox(3*643.7376)));
			drawString(". 300 milhas", 0);
			glRasterPos2f(eixoX(180, Aprox(3*643.7376)), eixoY(180, Aprox(3*643.7376)));
			drawString(". 300 milhas", 0);
			glRasterPos2f(eixoX(270, Aprox(3*643.7376)), eixoY(270, Aprox(3*643.7376)));
			drawString(". 300 milhas", 0);
			break;
		case KILOMETROS:
		default:
			glColor3f(0.5, 0.5, 0.5);
			// desenha os circulos marcando dada distncia
			for (unsigned r = 400; r <= 2000; r = r+400) {
				glBegin(GL_LINE_LOOP);
				for (unsigned int t = 0; t <360; t++) {
						glVertex3f(eixoX(t, r), eixoY(t, r), 0.0);
				} // for (unsigned int t = 0; t <360; t++)
				glEnd();
			} // for (unsigned r = 500; r <= 2000; r = r+500) 

			glColor3f(0.25, 0.25, 0.25);
			glRasterPos2f(eixoX(0, 400), eixoY(0, 400));
			drawString(". 100 km", 0);
			glRasterPos2f(eixoX(90, 400), eixoY(90, 400));
			drawString(". 100 km", 0);
			glRasterPos2f(eixoX(180, 400), eixoY(180, 400));
			drawString(". 100 km", 0);
			glRasterPos2f(eixoX(270, 400), eixoY(270, 400));
			drawString(". 100 km", 0);
			glRasterPos2f(eixoX(30, 800), eixoY(30, 800));
			drawString(". 200 km", 0);
			glRasterPos2f(eixoX(150, 800), eixoY(150, 800));
			drawString(". 200 km", 0);
			glRasterPos2f(eixoX(210, 800), eixoY(210, 800));
			drawString(". 200 km", 0);
			glRasterPos2f(eixoX(330, 800), eixoY(330, 800));
			drawString(". 200 km", 0);
			glRasterPos2f(eixoX(0, 1200), eixoY(0, 1200));
			drawString(". 300 km", 0);
			glRasterPos2f(eixoX(90, 1200), eixoY(90, 1200));
			drawString(". 300 km", 0);
			glRasterPos2f(eixoX(180, 1200), eixoY(180, 1200));
			drawString(". 300 km", 0);
			glRasterPos2f(eixoX(270, 1200), eixoY(270, 1200));
			drawString(". 300 km", 0);
			glRasterPos2f(eixoX(60, 1600), eixoY(60, 1600));
			drawString(". 400 km", 0);
			glRasterPos2f(eixoX(120, 1600), eixoY(120, 1600));
			drawString(". 400 km", 0);
			glRasterPos2f(eixoX(240, 1600), eixoY(240, 1600));
			drawString(". 400 km", 0);
			glRasterPos2f(eixoX(300, 1600), eixoY(300, 1600));
			drawString(". 400 km", 0);
			break;
	}
} // void desenhaEscala()













void drawSelecao(float x0, float y0, float x1 , float y1){
	/* desenha um retangulo, pode mudar a figura por aqui */
	glLineStipple(1,0xCCCC);
	glEnable(GL_LINE_STIPPLE);





	glLineWidth(1.0);
	glColor3f(0.0, 0.0, 0.0);
	if(figure.kind == 1) {
		glBegin(GL_LINE_LOOP); // desenha o retangulo
			glVertex3f(x0, y0, 0.0);
			glVertex3f(x0, y1, 0.0);
			glVertex3f(x1, y1, 0.0);
			glVertex3f(x1, y0, 0.0);
		glEnd();
	}
	else if (figure.kind == 2) {
		int raio0 = Aprox(RAIO*sqrt( x0*x0 + y0*y0));
		int	angulo0 = (360 + Aprox(atan2(y0, x0)*180 / PI))%360;
		int raio1 = Aprox(RAIO*sqrt( x1*x1 + y1*y1));
		int	angulo1 = (360 + Aprox(atan2(y1, x1)*180 / PI))%360;
		if(((360 + angulo0 - angulo1) %360) <((360 + angulo1 - angulo0) %360)) {
			unsigned int k = angulo0;
			angulo0 = angulo1;
			angulo1 = k;
		}
		int inteAng = (360 + angulo1 - angulo0)%360;





		glBegin(GL_LINE_STRIP);
			for(int i = 0; i < inteAng; i++) {
				int j = (360+angulo0+i)%360;
				glVertex3f(eixoX(j, raio0), eixoY(j, raio0), 0.0);
				glVertex3f(eixoX((j+1)%360, raio0), eixoY((j+1)%360, raio0), 0.0);
				}
		glEnd();
		
		glBegin(GL_LINE_STRIP);
			glVertex3f(eixoX(angulo1, raio0), eixoY(angulo1, raio0), 0.0);
			glVertex3f(eixoX(angulo1, raio1), eixoY(angulo1, raio1), 0.0);
		glEnd();

		glBegin(GL_LINE_STRIP);
			for(int i = 0; i < inteAng; i++) {
				int j = (360+angulo0+i)%360;
				glVertex3f(eixoX(j, raio1), eixoY(j, raio1), 0.0);
				glVertex3f(eixoX((j+1)%360, raio1), eixoY((j+1)%360, raio1), 0.0);
				}
		glEnd();

		glBegin(GL_LINE_STRIP);
			glVertex3f(eixoX(angulo0, raio1), eixoY(angulo0, raio1), 0.0);
			glVertex3f(eixoX(angulo0, raio0), eixoY(angulo0, raio0), 0.0);
		glEnd();
	}
	else if (figure.kind == 3) {
		x0 = x0/escalaGlobal + xZero;
		x1 = x1/escalaGlobal + xZero;
		y0 = y0/escalaGlobal + yZero;
		y1 = y1/escalaGlobal + yZero;

		if (isLinha == true) {
			unsigned int angulo = ((360 + Aprox(atan2(y0, x0)*180 / PI))%360);
			unsigned int raio0 = Aprox(RAIO*sqrt( x0*x0 + y0*y0));
			unsigned int raio1 = Aprox(RAIO*sqrt( x1*x1 + y1*y1));
			




			if (raio0 < matriz.getRaioInicial())
				raio0 = matriz.getRaioInicial();
			else if (raio0 > matriz.getRaioFinal())
				raio0 = matriz.getRaioFinal();






			if (raio1 > matriz.getRaioFinal())
				raio1 = matriz.getRaioFinal();
			else if (raio1 < matriz.getRaioInicial())
				raio1 = matriz.getRaioInicial();
			




			if (raio0 == raio1) {
				raio0 = matriz.getRaioInicial();
				raio1 = matriz.getRaioFinal();
			}





			glPushMatrix();
				glTranslatef(-xZero*escalaGlobal, -yZero*escalaGlobal, 0.0);
				glScalef(escalaGlobal, escalaGlobal, 1.0);
				glBegin(GL_LINE_STRIP);
					glVertex3f(eixoX(angulo, raio0), eixoY(angulo, raio0), 0.0);
					glVertex3f(eixoX(angulo, raio1), eixoY(angulo, raio1), 0.0);
				glEnd();
			glPopMatrix();

			}
		else {
			unsigned int angulo0 = ((360 + Aprox(atan2(y0, x0)*180 / PI))%360);
			unsigned int angulo1 = ((360 + Aprox(atan2(y1, x1)*180 / PI))%360);
			unsigned int raio = Aprox(RAIO*sqrt( x0*x0 + y0*y0));

			if (raio < matriz.getRaioInicial())
				raio = matriz.getRaioInicial();
			else if (raio > matriz.getRaioFinal())
				raio = matriz.getRaioFinal();

			if ( (360 + angulo0 - angulo1)%360 <  (360 + angulo1 - angulo0)%360 ) {
				unsigned int k = angulo0;
				angulo0 = angulo1;
				angulo1 = k;
			}
			int inteAng = (360 + angulo1 - angulo0)%360;

			glPushMatrix();
				glTranslatef(-xZero*escalaGlobal, -yZero*escalaGlobal, 0.0);
				glScalef(escalaGlobal, escalaGlobal, 1.0);
				glBegin(GL_LINE_STRIP);
				for(int i = 0; i < inteAng; i++) {
					int j = (360+angulo0+i)%360;
					glVertex3f(eixoX(j, raio), eixoY(j, raio), 0.0);
					glVertex3f(eixoX((j+1)%360, raio), eixoY((j+1)%360, raio), 0.0);
					}
				glEnd();
			glPopMatrix();

			}
			x0 = x0*escalaGlobal - xZero*escalaGlobal;
			x1 = x1*escalaGlobal - xZero*escalaGlobal;
			y0 = y0*escalaGlobal - yZero*escalaGlobal;
			y1 = y1*escalaGlobal - yZero*escalaGlobal;

		}




	glDisable(GL_LINE_STIPPLE);
}










void drawCores() {
	char numero[6];
	switch (coresSuave) {
		case COR_MAXIMO:
			for (int i = 0; i < 32; i++) {
				drawQuadradoSuave(-0.2, i/32.0 - 0.8, 0.4, 0.03125, i); 
				glColor3f(1.0, 1.0, 1.0);
				sprintf(numero, "%+5.1f", i*2.5-12.5);
				glRasterPos2d(-0.6, i/32.0 - 0.8);
				drawString(numero, 0);





				glColor3ub(cores[i].R, cores[i].G, cores[i].B);
				sprintf(numero, "%+5.1f", i*2.5-12.5);
				glRasterPos2d(+0.25, i/32.0 - 0.8);
				drawString(numero, 0);
			}
			break;
		case COR_MEDIO:
			for (int i = 0; i < 32; i++) {
				drawQuadradoReal(-0.2, i/32.0 - 0.8, 0.4, 0.03125, i); 
				glColor3f(1.0, 1.0, 1.0);
				sprintf(numero, "%+5.1f", i*2.5-12.5);
				glRasterPos2d(-0.6, i/32.0 - 0.8);
				drawString(numero, 0);





				glColor3ub(cores[i].R, cores[i].G, cores[i].B);
				sprintf(numero, "%+5.1f", i*2.5-12.5);
				glRasterPos2d(+0.25, i/32.0 - 0.8);
				drawString(numero, 0);
			}
			break;
		case COR_MINIMO:
			for (int i = 0; i < 32; i++) {
				int tmpcor;





				if (i < 5)
					tmpcor = 1;
				else if (i < 15)
					tmpcor = 13;
				else if (i < 23)
					tmpcor = 16;
				else if (i < 29)
					tmpcor = 26;
				else tmpcor = 30;





				drawQuadradoReal(-0.2, i/32.0 - 0.8, 0.4, 0.03125, tmpcor); 
				glColor3f(1.0, 1.0, 1.0);
				sprintf(numero, "%+5.1f", i*2.5-12.5);
				glRasterPos2d(-0.6, i/32.0 - 0.8);
				drawString(numero, 0);





				glColor3ub(cores[tmpcor].R, cores[tmpcor].G, cores[tmpcor].B);
				sprintf(numero, "%+5.1f", i*2.5-12.5);
				glRasterPos2d(+0.25, i/32.0 - 0.8);
				drawString(numero, 0);
			}
			break;
		case COR_BRILHO:
			for (int i = 0; i < 32; i++) {
				glShadeModel(GL_SMOOTH);
				glBegin(GL_POLYGON);
					glColor3f(0.0, 0.0, 0.0);
					glVertex3f(-0.2, -0.8, 0.0);
					glVertex3f(0.2, -0.8, 0.0);
					glColor3f(1.0, 1.0, 1.0);
					glVertex3f(0.2, 0.2, 0.0);
					glVertex3f(-0.2, 0.2, 0.0);
				glEnd();
				glShadeModel(GL_FLAT);




				glColor3f(1.0, 1.0, 1.0);
				glBegin(GL_LINE_LOOP);
					glVertex3f(-0.2, -0.8, 0.0);
					glVertex3f(0.2, -0.8, 0.0);
					glVertex3f(0.2, 0.2, 0.0);
					glVertex3f(-0.2, 0.2, 0.0);
				glEnd();





				glColor3f(1.0, 1.0, 1.0);
				sprintf(numero, "%+5.1f", i*2.5-12.5);
				glRasterPos2d(-0.6, i/32.0 - 0.8);
				drawString(numero, 0);




				glColor3ub(i*8, i*8, i*8);
				sprintf(numero, "%+5.1f", i*2.5-12.5);
				glRasterPos2d(+0.25, i/32.0 - 0.8);
				drawString(numero, 0);
			}
			break;
	}

}









void drawQuadradoReal(float x, float y, float xp, float yp, int cor) {
	// desenha quadrado colorido
	glColor3ub(cores[cor].R, cores[cor].G, cores[cor].B);
	glBegin(GL_POLYGON);
		glVertex3f(x, y, 0.0);
		glVertex3f(x + xp, y, 0.0);
		glVertex3f(x + xp, y + yp, 0.0);
		glVertex3f(x, y + yp, 0.0);
	glEnd();




	// desenha contorno
	/*
	glLineWidth(1.0);
	glColor3f(0.0, 0.0, 0.0);
	glBegin(GL_LINE_LOOP);
		glVertex3f(x, y, 0.0);
		glVertex3f(x + xp, y, 0.0);
		glVertex3f(x + xp, y + yp, 0.0);
		glVertex3f(x, y + yp, 0.0);
	glEnd();*/
}







void drawQuadradoSuave(float x, float y, float xp, float yp, int cor) {
	// desenha quadrado colorido
	glShadeModel(GL_SMOOTH);
	glBegin(GL_POLYGON);
		glColor3ub(cores[cor].R, cores[cor].G, cores[cor].B);
		glVertex3f(x, y, 0.0);
		glVertex3f(x + xp, y, 0.0);
		glColor3ub(cores[cor+1].R, cores[cor+1].G, cores[cor+1].B);
		glVertex3f(x + xp, y + yp, 0.0);
		glVertex3f(x, y + yp, 0.0);
	glEnd();
	glShadeModel(GL_FLAT);
}













/*******************************************************************************
 *******************************************************************************
 *******************************************************************************
            Controle de Teclado
 *******************************************************************************
 *******************************************************************************
 *******************************************************************************/








void teclado(unsigned char key, int x, int y) {
	x++;
	y++;
	switch (key) {
		case 27:
			estadoTemporario = estado;
			estado = estadoAnterior;
			estadoAnterior = estadoTemporario;
			escalaGlobal = escalaGlobalOld;
			xZero = xZeroOld;
			yZero = yZeroOld;




			estadoAnterior = ESPERA;
			escalaGlobalOld = 1.0;
			xZeroOld = 0.0;
			yZeroOld = 0.0;





			glutPostRedisplay();
			break;
		case 'x':
			switch (coresSuave) {
				case COR_BRILHO:
					coresSuave = COR_MINIMO;
					break;
				case COR_MINIMO:
					coresSuave = COR_MEDIO;
					break;
				case COR_MEDIO:
					coresSuave = COR_MAXIMO;
					break;
				case COR_MAXIMO:
					coresSuave = COR_BRILHO;
				}
			glutSetWindow(janelaPrincipal);
			glutPostRedisplay();
			glutSetWindow(janelaMenu);
			glutPostRedisplay();
			break;
		}
}













void keyboard(int key, int x, int y) {
	mousePosicao(x, y);




	switch (estado) {
		case EXIBE_GRAFICO:
			
			if (isLinha == true) {
				switch (key) {
					case GLUT_KEY_LEFT:
						ajusteR--;
						break;
					case GLUT_KEY_UP:
						ajusteA++;
						break;
					case GLUT_KEY_RIGHT:
						ajusteR++;
						break;
					case GLUT_KEY_DOWN:
						ajusteA--;
							break;
					default:
						break;
				}
				ajusteA = ajusteA%360;
			}
			else {
				switch (key) {
					case GLUT_KEY_LEFT:
						ajusteA--;
						break;
					case GLUT_KEY_UP:
						ajusteR++;
						break;
					case GLUT_KEY_RIGHT:
						ajusteA++;
						break;
					case GLUT_KEY_DOWN:
						ajusteR--;
							break;
					default:
						break;
				}	
				ajusteR = ajusteR%360;
			}
			break;



		case EXIBE_NUMEROS:
			switch (key) {
				case GLUT_KEY_LEFT:
					ajusteA--;
					break;
				case GLUT_KEY_UP:
					ajusteR--;
					break;
				case GLUT_KEY_RIGHT:
					ajusteA++;
					break;
				case GLUT_KEY_DOWN:
					ajusteR++;
						break;
				default:
					break;
			}		
			
			ajusteA = ajusteA%360;
			break;
		case EXIBE_ZOOM:
		case SELECIONA_ZOOM:
		case SELECIONA_ZOOM_DE_CORTE:
		case EXIBE_FATIA:
		case SELECIONA_GRAFICO:
		case SELECIONA_CENTRO:
		case SELECIONA_CORTE_DE_ZOOM:
		case SELECIONA_NUMEROS:
			if (escalaGlobal > 1.0) {
				switch (key) {
					case GLUT_KEY_LEFT:
						xZero -= 0.00025 * escalaGlobal;
						break;
					case GLUT_KEY_UP:
						yZero += 0.00025 * escalaGlobal;
						break;
					case GLUT_KEY_RIGHT:
						xZero += 0.00025 * escalaGlobal;
						break;
					case GLUT_KEY_DOWN:
						yZero -= 0.00025 * escalaGlobal;
						break;
					default:
						break;
				}
			}
			break;
		default: 
			break;
	}





	glutSetWindow(janelaPrincipal);
	glutSwapBuffers();
	glutPostRedisplay();
	glutSetWindow(janelaMenu);
	glutSwapBuffers();
	glutPostRedisplay();
}













/*******************************************************************************
 *******************************************************************************
 *******************************************************************************
            Sem relao
 *******************************************************************************
 *******************************************************************************
 *******************************************************************************/








char *imageName() {
	char *nomeTmp = matriz.getNome();
	int tamanho = strlen(nomeTmp)+10;
	char *tmpFinal;




	tmpFinal = new char [tamanho];
	// Basicamente, adiciona-se um nmero sequencial no nome, e a extenso .tif
	sprintf(tmpFinal, "%s-%04i.tif", nomeTmp, contadorDeImagens++);
	return tmpFinal;
}













void selecaoInicia(float x, float y) {
	contaPontos = 1;
	contaPontosReta = 1;
	xSelecaoAtual = x;
	ySelecaoAtual = y;
	xSelecaoInicial = x;
	ySelecaoInicial = y;
	tracking = true;
}












void selecaoTermina(float x, float y) {
	/* salva os dados no temporario figure */
	xSelecaoAtual = x;
	ySelecaoAtual = y;
	tracking = false; /* apaga o retangulo */
	contaPontos = 2;
	contaPontosReta = 2;
	figure.x0 = xSelecaoInicial;
	figure.y0 = ySelecaoInicial;
	figure.x1 = xSelecaoAtual;
	figure.y1 = ySelecaoAtual;
	glutPostRedisplay();
}









void resetFigure() {
	figure.kind = 0;
	figure.x0 = 0;
	figure.y0 = 0;
	figure.x1 = 0;
	figure.y1 = 0;
}









